/* ----------------------------------------------------
  Program      : 07_03_Storage.cpp
  Date			: Aug 16, 2001
  Version		: 1.0

  Description: This program illustrate storage formats for
  signed and unsigned integers, doubles and floats.  

  Some hard-coded values for each of the four types are
  displayed as decimal values and as stored hexadecimal
  values.  For purposes of comparison with debugger
  output the addresses of each of these quantities is
  also displayed.
   --------------------------------------------------- */
#include <iostream>
#include <iomanip>
#include <fstream>
using namespace std;

   // Structures used to interpret real numbers as both
   // floating point quantities and as integers.
 union stor4
  { float    x;
    unsigned i;
    char     c[4];
  };

 union stor8
  { double    x;
    unsigned  i[2];
  };


   // Overloaded output functions
void Display(unsigned u[], int size);
void Display(int      s[], int size);
void Display(stor4    f[], int size);
void Display(stor8    d[], int size);


ofstream ofil;


int main()
{  int i;

  // Open and verify output file
  ofil.open("o.dat");
  if (!ofil)
	 {  cout << "Could not open o.dat" << endl;
        return 1;
	 }

    // Assign values to arrays of unsiged and signed integers
  unsigned  u[5] = {  1, 27,  5635, 2123456789, 4123456789};
  int       s[5] = { -1,-27, -385, 2123456789, 4123456789};
  
	// Assign values to arrays of floats and doubles
  stor4  f[5];

//  f[0].x = 1.5F;
//  f[1].x = 504.21F;         // 504.21      = 1F8.35C28F5C2 
//  f[2].x = -504.21F;
//  f[3].x = 3.12345678e7F;   // 31234567.8  = 1DC9A07.AAAA

  f[0].x = -89.375F;
  f[1].x = 1024.5F;         
  strcpy(f[2].c,"efaC");
  f[3].x = -345.675F;   
  f[4].x = 2345678.;

  f[4].x = f[3].x;
  for (i = 0; i<10000; ++i) // Should not affect value since
     f[4].x -=  1.0F;		 // precision is only six digits

  stor8  d[5];
  d[0].x = 1.5;
  d[1].x = 504.21;
  d[2].x = -504.21;
  d[3].x = 3.12345678e7;
  d[4].x = d[3].x;
  for (i = 0; i<10000; ++i)
     d[4].x -=  1.0;


	// Output the identifier, decimal value, address and storage form
    // for each of the declared variables.

  ofil << " Name          decimal     address           hex  \n";
  ofil << " ----------------------------------------------------\n";

  Display(u,5);
  Display(s,5);
  Display(f,5);
  Display(d,5);

  ofil << " ----------------------------------------------------\n\n"; 
  return 0;
}





void Display(unsigned u[], int size)
/* ----------------------------------------------------------
   Display the array index, the decimal value, the address
   and the storage form in hexadecimal of each unsigned integer.
   ---------------------------------------------------------- */
{
  for (int i=0; i<size; ++i)
   ofil << "  u[" << i << "] "
        << setw(16) << dec << u[i] 
	    << setw(12) << &u[i] 
	    << setw(12) << hex << u[i] << endl;

  ofil << endl;
}



void Display(int s[], int size)
/* ----------------------------------------------------------
   Display the array index, the decimal value, the address
   and the storage form in hexadecimal of each signed integer.
   ---------------------------------------------------------- */
{
  for (int i=0; i<size; ++i)
   ofil << "  s[" << i << "] "
        << setw(16) << dec << s[i] 
	    << setw(12) << &s[i] 
	    << setw(12) << hex << s[i] << endl;

  ofil << endl;
}



void Display(stor4 f[], int size)
/* ----------------------------------------------------------
   Display the array index, the decimal value, the address
   and the storage form in hexadecimal of each 4-byte real number.
   ---------------------------------------------------------- */
{
  for (int i=0; i<size; ++i)
    ofil << "  f[" << i << "] "
         << setw(16) << fixed << showpoint << f[i].x 
	     << setw(12) << &f[i]
		 << setw(12) << hex <<  f[i].i << endl;

  ofil << endl;
}




void Display(stor8 d[], int size)
/* ----------------------------------------------------------
   Display the array index, the decimal value, the address
   and the storage form of each 8-byte real number.  The
   storage form is obtained by displaying the eight bytes
   as two 4-byte integers in hexadecimal form.
   ---------------------------------------------------------- */
{
  for (int i=0; i<size; ++i)
    ofil << "  d[" << i << "] " 
	     << setw(16) << fixed << showpoint << d[i].x 
	     << setw(12)  << &d[i]      << "    "
	     << hex << d[i].i[1] << ' ' << d[i].i[0] << endl;

  ofil << endl;
}
