/* ----------------------------------------------------
  Program      : Check_Storage.cpp
  Date			: Apr 8, 2004
  Version		: 2.0

  Description: This program illustrates storage formats for
  signed and unsigned integers, doubles and floats.  

  Some user-supplied values for each of the four types are
  displayed as decimal values and as stored hexadecimal
  values.  For purposes of comparison with debugger
  output the addresses of each of these quantities is
  also displayed.
   --------------------------------------------------- */
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <cctype>
using namespace std;

   // Structures used to interpret real numbers as both
   // floating point quantities and as integers.
 union stor4
  { float    x;
    unsigned i;
    char     c[4];
  };

 union stor8
  { double    x;
    unsigned  i[2];
  };



void AssignUInts(unsigned u[], int size);
void AssignSInts(int s[], int size);
void AssignFloats(stor4 f[], int size);
void AssignDoubles(stor8 d[], int size);

   // Overloaded output functions
void Display(unsigned u[], int size);
void Display(int      s[], int size);
void Display(stor4    f[], int size);
void Display(stor8    d[], int size);

bool GetResponse(string prompt);
void OverflowTest();


ofstream ofil;


int main()
{  
   unsigned u[5];
   int      s[5];
   stor4    f[5];
   stor8    d[5];

  // Open and verify output file
  ofil.open("o.dat");
  if (!ofil)
	 {  cout << "Could not open o.dat" << endl;
        return 1;
	 }

  cout << "\n\nYou will be prompted to enter five each of the "
       << " following: \n"
       << " unsigned integers\n"
       << " signed integers\n"
       << " floating point values\n"
       << " double values\n\n"
       << "You may elect to use default values instead\n\n";

  cout << "After the values are entered (accepted), they will\n"
       << " be written to o.dat in fixed point decimal and as\n"
       << " hexadecimal values.\n\n";

    // Assign values to arrays of unsiged and signed integers
  AssignUInts(u,5);
  AssignSInts(s,5);
  
	// Assign values to arrays of floats and doubles
  AssignFloats(f,5);
  AssignDoubles(d,5);


	// Output the identifier, decimal value, address and storage form
    // for each of the declared variables.

  ofil << " Name          decimal     address           hex  \n";
  ofil << " ----------------------------------------------------\n";

  Display(u,5);
  Display(s,5);
  Display(f,5);
  Display(d,5);

  ofil << " ----------------------------------------------------\n\n"; 

  OverflowTest();
  return 0;
}


void AssignUInts(unsigned u[], int size)
{
   // Assign default values
  u[0] = 1;
  u[1] = 27;
  u[2] = 5635;
  u[3] = 2123456789;
  u[4] = 4123456789;

  if ( GetResponse("\nUse default values for unsigned ints?") )
    return;

  cout << "\nEnter five unsigned integers\n";
  for (int i=0; i<size; ++i)
     cin >> u[i];
}


void AssignSInts(int s[], int size)
{
   // Assign default values
  s[0] = -1;
  s[1] = -27;
  s[2] = -385;
  s[3] = 2123456789;
  s[4] = 4123456789;

  if ( GetResponse("\nUse default values for signed ints?") )
    return;

  cout << "\nEnter five signed integers\n";
  for (int i=0; i<size; ++i)
     cin >> s[i];
}



void AssignFloats(stor4 f[], int size)
{
//  f[0].x = 1.5F;
//  f[1].x = 504.21F;         // 504.21      = 1F8.35C28F5C2 
//  f[2].x = -504.21F;
//  f[3].x = 3.12345678e7F;   // 31234567.8  = 1DC9A07.AAAA

  f[0].x = -89.375F;
  f[1].x = 1024.5F;         
  strcpy(f[2].c,"efaC");
  f[3].x = -345.675F;   
  f[4].x = 2345678.;

  if ( GetResponse("\nUse default values for floats? ") )
    return;

  cout << "\n\nFloats may be entered as floating point values\n"
       << " or as strings of four characters\n\n";
  if ( GetResponse("\nEnter values as floats? ") )
   {
      cout << "\nEnter five floats\n";
      for (int i=0; i<size; ++i)
         cin >> f[i].x;
   }
  else
   {
      cout << "\nEnter five four-character strings\n";
      for (int i=0; i<size; ++i)
         cin >> f[i].c;
   }
}


void AssignDoubles(stor8 d[], int size)
{
   // Assign default values
  d[0].x = 1.5;
  d[1].x = 504.21;
  d[2].x = -504.21;
  d[3].x = 3.12345678e7;
  d[4].x = 6.023e23;


  if ( GetResponse("\nUse default values for doubles?") )
    return;

  cout << "\nEnter five double values\n";
  for (int i=0; i<size; ++i)
     cin >> d[i].x;

}



void Display(unsigned u[], int size)
/* ----------------------------------------------------------
   Display the array index, the decimal value, the address
   and the storage form in hexadecimal of each unsigned integer.
   ---------------------------------------------------------- */
{
  for (int i=0; i<size; ++i)
   ofil << "  u[" << i << "] "
        << setw(16) << dec << u[i] 
	    << setw(12) << &u[i] 
	    << setw(12) << hex << u[i] << endl;

  ofil << endl;
}



void Display(int s[], int size)
/* ----------------------------------------------------------
   Display the array index, the decimal value, the address
   and the storage form in hexadecimal of each signed integer.
   ---------------------------------------------------------- */
{
  for (int i=0; i<size; ++i)
   ofil << "  s[" << i << "] "
        << setw(16) << dec << s[i] 
	    << setw(12) << &s[i] 
	    << setw(12) << hex << s[i] << endl;

  ofil << endl;
}



void Display(stor4 f[], int size)
/* ----------------------------------------------------------
   Display the array index, the decimal value, the address
   and the storage form in hexadecimal of each 4-byte real number.
   ---------------------------------------------------------- */
{
  for (int i=0; i<size; ++i)
    ofil << "  f[" << i << "] "
         << setw(16) << fixed << showpoint << f[i].x 
	     << setw(12) << &f[i]
		 << setw(12) << hex <<  f[i].i << endl;

  ofil << endl;
}




void Display(stor8 d[], int size)
/* ----------------------------------------------------------
   Display the array index, the decimal value, the address
   and the storage form of each 8-byte real number.  The
   storage form is obtained by displaying the eight bytes
   as two 4-byte integers in hexadecimal form.
   ---------------------------------------------------------- */
{
  for (int i=0; i<size; ++i)
    ofil << "  d[" << i << "] " 
	     << setw(16) << fixed << showpoint << d[i].x 
	     << setw(12)  << &d[i]      << "    "
	     << hex << d[i].i[1] << ' ' << d[i].i[0] << endl;

  ofil << endl;
}


bool GetResponse(string prompt)
// The prompt must ask a question with a y/n answer
{ char response;

  cout << prompt << endl;
  cout << "\nEnter y or n: ";
  cin >> response;

  if ( toupper(response) == 'Y')
    return true;
  else
    return false;
}


void OverflowTest( )
{ int i;
  stor4 f[2];
  stor8 d[2];

  cout << "\n\nOverflow test: 1.0 will be subtracted from a \n"
       << " floating point value 10000 times, but the value will\n"
       << " be unchanged due to representational error.\n"
       << " The process will be repeated with a double value\n"
       << " which will reflect the subtractions\n\n";

  f[0].x = 3.12345678e7F;
  f[1].x = f[0].x;

  cout << fixed << showpoint << setprecision(6);
  cout << "f[0] = " << f[0].x << endl;


  for (i = 0; i<10000; ++i) // Should not affect value since
     f[1].x -=  1.0F;		 // precision is only six digits
  cout << "\nAfter subtracting 1 from f[0], 10000 times: " 
       << f[1].x << endl;

  Display(f,2);


  d[0].x = 3.12345678e7;
  d[1].x = d[0].x;

  cout << "d[0] = " << d[0].x << endl;


  for (i = 0; i<10000; ++i) 
     d[1].x -=  1.0;		 
  cout << "\nAfter subtracting 1 from d[0], 10000 times: " 
       << d[1].x << endl;

  Display(d,2);

}