/* ------------------------------------------------------------------
	Program	: 08_34_Phones.cpp
	Date	   : Apr 8, 2004
    Topic	: Modifying Objects

  Description: this program simulates the processing of a phone number
  by the telephone company.

  Input:  phoneNumber.
  Output: areaCode, exchange and localNumber.
 
  Discuss:
	Strings
	String operations--substr, find_first_not_of
 ------------------------------------------------------------------ */

#include <iostream>              
#include <string> 
#include <cstdlib>                     
using namespace std;                 

void DisplayFormattedNumber(string phoneNumber);
bool IsValid(string phoneNumber);
void PhoneError(string message);



int main()
{
 string phoneNumber;

  cout << "Enter phone numbers with digits only (no spaces, hyphens):\n";
  cout << "Examples:  7421757     12145067823 \n";
  cout << "\nEnter a phone number: ";
  cin >> phoneNumber;
  cout << endl << endl;

  if ( IsValid(phoneNumber) )
	  DisplayFormattedNumber(phoneNumber);


  return 0;
}



void DisplayFormattedNumber(string phoneNumber)
/*
  Discard the initial 0,1 for long distance calls, then
  display the exchange and the local number.
 ------------------------------------------------------ */
{
  if (phoneNumber[0] == '1')
    cout << '(' << phoneNumber.substr(1, 3) << ")-"
         << phoneNumber.substr(4, 3) << '-'
         << phoneNumber.substr(7, 4) << endl;
  else
     cout << phoneNumber.substr(0, 3) << '-'
          << phoneNumber.substr(3, 4) << endl;
}



bool IsValid(string s)
/* CheckValidity ensures that a string is a valid phone number.
  
  Receive: s, a string
  Return:  true if s is a valid phone number
  
  The parameter s is a valid phone number if it consists of
  7 or 11 digits.   It 11 digits in length, the first digit
  must be a '1'.

  Note: check VC++ Help to learn more about the members of
  the string class.   In particular, read about the
  find_first_not_of() method.
 ------------------------------------------------------------ */
{
		// check number of digits
   if (s.size() != 7   &&   s.size() != 11)
   {	PhoneError("A phone number has 7 or 11 digits");
		return false;
	}
		// check first digit
   if (s.size() == 7 && (s[0] == '0' || s[0] == '1'))
	  {  PhoneError("A local call cannot begin with 0 or 1");
         return false;
	  }
   if (s.size() == 11 && s[0] != '1')
	  {  PhoneError("A long distance call must begin with 1");
         return false;
	  }
   if (s.find_first_not_of("0123456789", 0) != string::npos)
   {     PhoneError("A phone number must consist of all digits");
         return false;
   }
   return true;
}



void PhoneError(string message)
/* PhoneError displays an error msg 
  
  Receive: message, a string.
  Output:  message, with error-specific text  
------------------------------------------------------------ */
{
   cerr << "Your call cannot be copleted because:\n"
        << message <<  ".\nPlease try again." << endl;   
}

