// 20_05_InsSort.cpp

#include <iostream>
#include <iomanip>
#include <vector>
#include <string>
#include "20_05_RandomInt.h"
#include "20_05_Employee.h"
using namespace std;


void	 FillArray(int      values[], int size, int len, bool unique);
void	 FillArray(double   values[], int size, int len, bool unique);
void	 FillArray(string   values[], int size, int len, bool unique);
void	 FillArray(Employee values[], int size, int len, bool unique);
string GenerateRandomString(int size, bool upper);


template <typename T>
void insertionSort( T a[], int size );

template <typename T>
void Print(T values[], int size, int spacing, int npl, string msg);


const int MAX = 100;

int main( )
{
   int		  v[MAX];
   double	  d[MAX];
   string   s[MAX];
   Employee e[MAX];

   FillArray(v, MAX, 9000, true);
   Print(v, MAX, 7, 10, "initial integer array: ");
   insertionSort(v, MAX );
   Print(v, MAX, 7, 10, "sorted integer array: ");
   cin.get();

   FillArray(d, MAX, 900, true);
   Print(d, MAX, 10, 5, "initial double array: ");
   insertionSort(d, MAX );
   Print(d, MAX, 10, 5, "sorted double array: ");
   cin.get();

   FillArray(s, MAX, 6, true);
   Print(s, MAX, 7, 10, "initial string array: ");
   cin.get();
   insertionSort(s, MAX );
   Print(s, MAX, 7, 10, "sorted string array: ");
   cin.get();


   FillArray(e, MAX/2, 6, true);
   Print(e, MAX/2, 20, 2, "initial employee array: ");
   cin.get();
   insertionSort(e, MAX/2 );
   Print(e, MAX/2, 20, 2, "sorted employee array: ");


    return 0;
}







////////////////////////////////////////////////////////////////////////
//////      Utilities
////////////////////////////////////////////////////////////////////////





void FillArray(int values[], int size, int maxValue, bool unique)
/* -----------------------------------------------------------------
   Generate random numbers between 1 and 'maxValue' until 'size'
   values have been stored.  
   If 'unique' is set to false, then duplicates are allowed.
   If 'unique' is set to true, then duplicates are discarded.
   ----------------------------------------------------------------- */
{ int  i, candidate, index = 0;  
  bool duplicate = false;
  RandomInt r(1,maxValue);

  cout << "\n\n Generating random integer array of size " << size << ": ";
  values[index++] = r.Generate();

  do
    {  candidate = r.Generate();
       
	   if (unique)
		 {  i = 0;
			while (i < index &&  candidate != values[i++]);
            duplicate =  i < index;
		 }
       if ( ! duplicate )
			values[index++] = candidate;   
       if ( index % 200 == 0 ) cout << '.';         
	} while ( index < size );
   
  cout << "    complete\n\n";
			
}



void FillArray(double values[], int size, int maxValue, bool unique)
/* -----------------------------------------------------------------
   Generate random numbers between 1 and 'maxValue' until 'size'
   values have been stored.  
   If 'unique' is set to false, then duplicates are allowed.
   If 'unique' is set to true, then duplicates are discarded.
   ----------------------------------------------------------------- */
{ int		i,index = 0; 
  double	candidate; 
  bool		duplicate = false;
  RandomInt r(1,maxValue);

  cout << "\n\n Generating random double array of size " << size << ": ";
  values[index++] = r.Generate()*10.0  +  double(r.Generate())/100.0;

  do
    {  candidate = r.Generate()*10.0  +  double(r.Generate())/100.0;
       
	   if (unique)
		 {  i = 0;
			while (i < index &&  candidate != values[i++]);
            duplicate =  i < index;
		 }
       if ( ! duplicate )
			values[index++] = candidate;   
       if ( index % 200 == 0 ) cout << '.';         
	} while ( index < size );
   
  cout << "    complete\n\n";
			
}



void FillArray(string values[], int size, int len, bool unique)
/* -------------------------------------------------------------------
   Generate random strings of length 'len' until 'size' values have 
   been stored.  
   If 'unique' is set to false, then duplicates are allowed.
   If 'unique' is set to true, then duplicates are discarded.   
   ------------------------------------------------------------------- */  
{ int    i, index = 0;  
  string candidate;
  bool   duplicate = false;

  cout << "\n\n Generating random string array of size " << size << ": ";
  values[index++] = GenerateRandomString(len, false);

  do
    {  candidate = GenerateRandomString(len, false);
       
	   if (unique)
		 {  i = 0;
			while (i < index &&  candidate != values[i++]);
            duplicate =  i < index;
		 }
       if ( ! duplicate )
			values[index++] = candidate;   
       if ( index % 200 == 0 ) cout << '.';         
	} while ( index < size );
   
  cout << "    complete\n\n";
			
}



void FillArray(Employee values[], int size, int len, bool unique)
/* -------------------------------------------------------------------
   Generate random Employee objects with names of length 'len' until
  'size' values have been stored.  
   ------------------------------------------------------------------- */  
{ int      i, index = 0;  
  Employee candidate;
  bool     duplicate;
  

  cout << "\n\n Generating random Employee array of size " << size << ": ";
  values[index++] = candidate.RandomFill(6);

  do
    {  candidate.RandomFill(6);
       
	   if (unique)
		 {  i = 0;
			while (i < index &&  candidate != values[i++]);
            duplicate =  i < index;
		 }
       if ( ! duplicate )
			values[index++] = candidate;   
       if ( index % 200 == 0 ) cout << '.';         
	} while ( index < size );
   
  cout << "    complete\n\n";
			
}

              



string GenerateRandomString(int size, bool upper)
/* -------------------------------------------------------------------
   Generate a string composed of random letters
   ------------------------------------------------------------------- */  
{
  char   *letters;
  string name;
  int    base = upper ? 0x41 : 0x61;

  letters = new char[size + 1];
  assert(letters);

  
  for (int i=0; i<size; ++i)
    letters[i] = char(rand()%26+base);

  letters[size] = '\0';
  name = letters;
  delete [] letters;

  return name;
  
}





template <typename T>
void Print(T values[], int size, int spacing, int npl, string msg)
/* -----------------------------------------------------------------
    Pre:  type T has an overloaded insertion operator.
	Post: -- 'msg' has been displayed 
		  --  all 'size' members of values[] have been printed with 
			  'npl' elements printed per line.   Each element is
			  displayed in a field of width 'spacing'
   ----------------------------------------------------------------- */
{ int  count = 0;

  cout << endl << msg << endl;
  cout << "\n Array has " << size << " elements" << endl;
       
  cout << fixed << setprecision(2);

  for (int i=0; i<size; ++i)
   {  
      cout << setw(spacing) << values[i];	  
      ++count;     
	  if ( count % npl == 0 )
        cout << endl;
   }
  cout << endl << endl;
}




template <typename T>
void insertionSort( T a[], int size )
// insertionSort: sort items in array a
// type T must have copy constructor, operator =, and operator <
{
    for( int p = 1; p < size; p++ )
    { int j;
      T   tmp = a[ p ];
            // Insert a[p] into sorted  a[0] ... a[p-1] by shifting
			      //   down by one all elements larger than a[p]
        for( j = p; j > 0 && tmp < a[ j - 1 ]; j-- )
            a[ j ] = a[ j - 1 ];
        a[ j ] = tmp;
    }
}
